@@ -27,9 +27,9 @@ import traceback
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.DEBUG)
 
-MSG_WARNING= ("Before you request bitcoins to be sent to addresses in this "
+MSG_WARNING= ("Before you request coins to be sent to addresses in this "
                     "wallet, ensure you can pair with your device, or that you have "
-                    "its seed (and passphrase, if any).  Otherwise all bitcoins you "
+                    "its seed (and passphrase, if any).  Otherwise all coins you "
                     "receive will be unspendable.")
                     
 MSG_USE_2FA= ("Do you want to use 2-Factor-Authentication (2FA)?\n\n"
@@ -695,13 +695,18 @@ class CardConnector:
             msg= msg+ [padsize]*padsize
             # send apdu
             (response, sw1, sw2) = self.card_transmit(apdu)
-            # extract IV & id_2FA
-            IV= response[0:16]
-            id_2FA= response[16:36]
-            msg_out=IV
-            # id_2FA is 20 bytes, should be 32 => use sha256
-            from hashlib import sha256
-            id_2FA= sha256(bytes(id_2FA)).hexdigest()
+            if sw1==0x90 and sw2==0x00:
+                # extract IV & id_2FA
+                IV= response[0:16]
+                id_2FA= response[16:36]
+                msg_out=IV
+                # id_2FA is 20 bytes, should be 32 => use sha256
+                from hashlib import sha256
+                id_2FA= sha256(bytes(id_2FA)).hexdigest()
+            elif sw1==0x9c and sw2==0x19:
+                raise RuntimeError(f"Error: 2FA is not enabled (error code: {hex(256*sw1+sw2)}")
+            else:
+                raise UnexpectedSW12Error(f'Unexpected error code: {hex(256*sw1+sw2)}')
         else:
             p1= 0x01
             lc= 0x10
@@ -714,7 +719,13 @@ class CardConnector:
                 logger.info('Padding error!')
             # send apdu
             (response, sw1, sw2) = self.card_transmit(apdu)
-
+            if sw1==0x90 and sw2==0x00:
+                pass
+            elif sw1==0x9c and sw2==0x19:
+                raise RuntimeError(f"Error: 2FA is not enabled (error code: {hex(256*sw1+sw2)}")
+            else:
+                raise UnexpectedSW12Error(f'Unexpected error code: {hex(256*sw1+sw2)}')
+            
         # msg is cut in chunks and each chunk is sent to the card for encryption/decryption
         # given the protocol overhead, size of each chunk is limited in size:
         # without secure channel, an APDU command is max 255 byte, so chunk<=255-(5+2)
@@ -841,7 +852,13 @@ class CardConnector:
                 msg = ("Too many failed attempts! Your Satochip has been blocked! You need your PUK code to unblock it.")
                 self.client.request('show_error', msg)
                 raise RuntimeError('Device blocked with error code:'+hex(sw1)+' '+hex(sw2))
-        
+            # any other edge case
+            else:
+                self.set_pin(0, None) #reset cached PIN value
+                msg = (f"Please check your card! \nUnexpected error sw12: {hex(sw1)} {hex(sw2)}")
+                self.client.request('show_error', msg)
+                return (response, sw1, sw2)     
+                
         #if not self.card_present:
         self.client.request('show_error', 'No Satochip found! Please insert card!')
         return
